package com.bull.cimero.pluginEditor.editors.model;

import it.eng.cimero.plugin.descriptor.PasswordTextPropertyDescriptor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.commands.ConnectionCreateCommand;

/**
 * This class is the parent class of all element of the model.
 * It contain all the fonctionnalities to add, remove connection beetween
 * element
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class GeneriqueCimeroModel extends Element
		implements IPropertySource {

	/** current diagram. */
	private Diagram diagram;
	/** serial UID. */
	static final long serialVersionUID = 1;
	/**To determine if a shell is currently open or not.*/
	private static boolean shellOpen = false;
	/**Boolean to determine if the name must be update.*/
	private boolean changeName = true;

	//
	//properties
	//
	/**The default in and out anchor to a component.*/
	public static final int DEFAULTOUTIN = 5;
	/**The location id.*/
	public static final String LOCATION = "location";
	/**The name id.*/
	public static final String NAME = "name";
	/**The inputs id.*/
	public static final String INPUTS = "inputs";
	/**The output id.*/
	public static final String OUTPUTS = "outputs";
	//
	//actual fields
	//
	/**The descriptor list.*/
	private transient List <IPropertyDescriptor> descriptorList = null;
	/**Map that contain all properties.*/
	private  Map<String, Object> propertiesMap =
								new HashMap < String, Object >();
	/**The source connection number.*/
	private int nbInputs = -1;
	/**The target connection number.*/
	private int nbOutputs = -1;
	/**The lovation of an element.*/
	private Point location = new Point(0, 0);
	/**The outputs connectioc list.*/
	private List < Connection > outputs = null;
	/**The input connection list.*/
	private List < Connection > inputs = null;
	/**to determine if the component is standard or not.*/
	private boolean standard = false;
	/** default name. */
	private String defaultName = "";
	
	private boolean customComponent = false;
	private String customComponentId  = null;

	/**
	 * copy constructor.
	 * @param nameParam : name to set
	 * @param nbOutputsParam : nbouput to set
	 * @param nbInputsParam : nbinput to set
	 */
	public GeneriqueCimeroModel(final String nameParam
			, final int nbOutputsParam, final int nbInputsParam) {
		this.location = new Point(0, 0);
		defaultName = nameParam;
		//add the property name
		setProperty(NAME, nameParam);
		setNbInputs(nbInputsParam);
		setNbOutputs(nbOutputsParam);
		//initialize the inputs and outputs list
		buildListInputsOutputs();
	}

	/**
	 * set location.
	 * @param p : location to set
	 */
	public final void setLocation(final Point p) {
		if (this.location.equals(p)) {
			return;
		}
		this.location = p;
		firePropertyChange(LOCATION, null, p);
	}

	/**
	 * return the location.
	 * @return Point
	 */
	public final Point getLocation() {
		return location;
	}



	/**
	 * Return the Object Name.
	 * @return String
	 */
	public final String getName() {
		String name = "";
		if (getProperty(NAME) != null) {
			name = (String) getProperty(NAME);
		}
		return name;
	}

	/**
	 * set the name value.
	 * @param name : the name value.
	 */
	public final void setName(final String name) {
		setProperty(NAME, name);
	}

	/**
	 * set the number of inputs elements.
	 * @param nb : nb inputs to set
	 */
	public final void setNbInputs(final int nb) {
		nbInputs = nb;
	}
	/**
	 * set the number of outputs elements.
	 * @param nb : nb outputs to set
	 */
	public final void setNbOutputs(final int nb) {
		nbOutputs = nb;
	}
	/**
	 * return the number of outputs elements. If the nbOutputs hasn't
	 * been specified, the default value is 5.
	 * @return int
	 */
	public final int getNbOutputs() {
		if (nbOutputs == -1) {
			return DEFAULTOUTIN;
		} else {
			return nbOutputs;
		}
	}
	/**
	 * return the number of inputs elements. If the nbInputs
	 * hasn't been specified, the default value is 5.
	 * @return int
	 */
	public final int getNbInputs() {
		if (nbInputs == -1) {
			return DEFAULTOUTIN;
		} else {
			return nbInputs;
		}
	}

	/**
	 * The diagram getter function.
	 * @return Diagram
	 */
	public final Diagram getDiagram() {
		return diagram;
	}

	/**
	 * the diagram setter function.
	 * @param diagramParam : diagram to set
	 */
	public final void setDiagram(final Diagram diagramParam) {
		this.diagram = diagramParam;
		//we verifiy that the sp?cified name not already exist.
		int indice = 1;
		if (changeName) {
			changeName = false;
			//set the name with the cimero diagram name in suffixe
			setProperty(NAME, getName()
					+ this.diagram.getCimeroDiagramName().split("\\.")[0]);
		}
		List lstTmp = this.diagram.getLstCimeroObject();
		for (int i = 0; i < lstTmp.size(); i++) {
			//if the current object has the same name of an other object
			if (getName().compareTo(
					((GeneriqueCimeroModel) lstTmp.get(i)).getName()
					) == 0) {
				//if the current object is different of the other object
				if (this != (GeneriqueCimeroModel) lstTmp.get(i))	{
					//we change the current object name
					setProperty(NAME, getDefaultName() + " " + indice);
					indice++;
					i = -1;
				}
			}
		}
		fireStructureChange(NAME, this);
	}

	/**
	 * @return the standard.
	 */
	public final boolean isStandard() {
		return standard;
	}

	/**
	 * @param standardP the standard to set
	 */
	public final void setStandard(final boolean standardP) {
		this.standard = standardP;
	}

	/**
	 * This function is used to create the list
	 * of output and inputs.
	 */
	private void buildListInputsOutputs() {
		//we delete all the inputs and outputs connection and
		//we rebuilt the list
		if (inputs != null) {
			removeAllInput();
		}
		if (outputs != null) {
			removeAllOutput();
		}
		inputs = new ArrayList < Connection >(getNbInputs());
		outputs = new ArrayList < Connection >(getNbOutputs());
	}


	//	------------------------------------------------------------------------
	// Function to add, delete and test the connection with this object

	/**
	 * Add a Input Connection to this object.
	 * @param connection : connection to add in input
	 * @return boolean
	 */
	public boolean addInput(final Connection connection) {
		if (this.inputs.size() < getNbInputs()) {
			this.inputs.add(connection);
			fireStructureChange(INPUTS, connection);
			return true;
		} else {
			Shell shell = new Shell();
			MessageDialog.openInformation(
					shell,
					"Exception",
			"Unable to add an other input connection");
			return false;
		}
	}

	/**
	 * Add an Output Connection to this Object.
	 * @param connection : connection to add in output
	 * @return boolean
	 */
	public final boolean addOutput(final Connection connection) {
		if (this.outputs.size() < getNbOutputs()) {
			this.outputs.add(connection);
			fireStructureChange(OUTPUTS, connection);
			return true;
		} else {
			Shell shell = new Shell();
			MessageDialog.openInformation(
					shell,
					"Exception",
			"Unable to add an other output connection");
			return false;
		}
	}

	/**
	 * Return all the Inputs Connections of this object.
	 * @return List
	 */
	public final List getIncomingConnections() {
		return this.inputs;
	}

	/**
	 * Return all the Output Connections of this object.
	 * @return List
	 */
	public final List getOutgoingConnections() {
		return this.outputs;
	}

	/**
	 * Delete the input connection of this Object.
	 * @param connection : input connection to remove
	 */
	public void removeInput(final Connection connection) {
		this.inputs.remove(connection);
		fireStructureChange(INPUTS, connection);
	}

	/**
	 * Delete the output connection of this object.
	 * @param connection : output connection to remove
	 */
	public final void removeOutput(final Connection connection) {
		this.outputs.remove(connection);
		fireStructureChange(OUTPUTS, connection);
	}

	/**
	 * This function is used to delete all Input connection.
	 *
	 */
	public final void removeAllInput() {
		for (int i = this.inputs.size() - 1; i >= 0; i--) {
			Connection myCon = (Connection) this.inputs.get(i);
			//on retire cette connection a celui de la source
			myCon.getSource().removeOutput(myCon);
			//on retire maintenant la connection de la liste
			this.inputs.remove(i);
			//on averti Eclipse d'un changement
			fireStructureChange(INPUTS, myCon);
		}
	}

	/**
	 * This function is used to delete all Output connection.
	 */
	public final void removeAllOutput() {
		for (int i = this.outputs.size() - 1; i >= 0; i--) {
			Connection myCon = (Connection) this.outputs.get(i);
			//on retire cette connection a celui de la destination
			myCon.getTarget().removeInput(myCon);
			//on retire maintenant la connection de la liste
			this.outputs.remove(i);
			fireStructureChange(OUTPUTS, myCon);
		}
	}

	/**
	 * @return true is the inputs list is not null.
	 */
	public final boolean isEntryConnection() {
		if (inputs.size() != 0) {
			return true;
		}
		return false;
	}

	/**
	 * @return true is the outputs list is not null
	 */
	public final boolean isOutputConnection() {
		if (outputs.size() != 0) {
			return true;
		}
		return false;
	}

	/**
	 * This function is used to known if a element name already exist
	 * in the list of component.
	 * @param nameParam : the name to search in the component list.
	 * @return true if the name exists and not if the name doesn't exist
	 */
	protected final boolean nameExist(final String nameParam) {
		if (diagram == null) {
			return false;
		}
		List < GeneriqueCimeroModel > lstDiagramObject =
			diagram.getLstCimeroObject();
		for (int i = 0; i < lstDiagramObject.size(); i++) {
			if (lstDiagramObject.get(i).getName().compareTo(nameParam) == 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * this function is used to retreive all the object
	 * in the model that containt an input connection.
	 * @return String[]
	 */
	protected final String [] getLstObjectDrawWithInputConnection() {
		//we retreive all the element of the model
		List lstDiagramObject = diagram.getLstCimeroObject();
		//we count the element which have input connection
		int nbEltWithInput = 0;
		for (int i = 0; i < lstDiagramObject.size(); i++) {
			if (((GeneriqueCimeroModel) lstDiagramObject.get(i)).isInput()) {
				if (((GeneriqueCimeroModel) lstDiagramObject
						.get(i)).getName().compareTo(getName()) != 0) {
					nbEltWithInput++;
				}
			}
		}

		String [] lstObject = new String[nbEltWithInput + 1];
		int indiceList = 1;
		lstObject[0] = "";
		for (int i = 0; i < lstDiagramObject.size(); i++) {
			if (((GeneriqueCimeroModel) lstDiagramObject.get(i)).isInput()) {
				if (((GeneriqueCimeroModel)
						lstDiagramObject.get(i)).
						getName().compareTo(getName()) != 0) {
					lstObject[indiceList] =
						((GeneriqueCimeroModel)
								lstDiagramObject.get(i)).getName();
					indiceList++;
				}
			}
		}
		return lstObject;
	}

	/**
	 * this function is used to retreive all the object
	 * in the model that containt an input connection.
	 * @return String[]
	 */
	protected final String [] getLstObjectDrawWithOutputConnection() {
		//we retreive all the element of the model
		List lstDiagramObject = diagram.getLstCimeroObject();
		//we count the element which have input connection
		int nbEltWithInput = 0;
		for (int i = 0; i < lstDiagramObject.size(); i++) {
			if (((GeneriqueCimeroModel) lstDiagramObject.get(i)).isOutput()) {
				if (((GeneriqueCimeroModel) lstDiagramObject
						.get(i)).getName().compareTo(getName()) != 0) {
					nbEltWithInput++;
				}
			}
		}

		String [] lstObject = new String[nbEltWithInput + 1];
		int indiceList = 1;
		lstObject[0] = "";
		for (int i = 0; i < lstDiagramObject.size(); i++) {
			if (((GeneriqueCimeroModel) lstDiagramObject.get(i)).isOutput()) {
				if (((GeneriqueCimeroModel)
						lstDiagramObject.get(i)).
						getName().compareTo(getName()) != 0) {
					lstObject[indiceList] =
						((GeneriqueCimeroModel)
								lstDiagramObject.get(i)).getName();
					indiceList++;
				}
			}
		}
		return lstObject;
	}
	
	/**
	 * This function return true if the connection in parameter exist in
	 * the outgoing connection list.
	 * @param conn : the connection
	 * @return boolean
	 */
	public final boolean isConnectionExistInOut(final Connection conn) {
		for (int i = 0; i < getOutgoingConnections().size(); i++) {
			Connection tmp = (Connection) getOutgoingConnections().get(i);
			if (tmp.equals(conn)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This function is used to create the connection beetween
	 * this object and the new destination object.
	 * @param nameDestination : the name of the destination element
	 * @return boolean
	 */
	protected boolean refreshDestinationConnection(
			final String nameDestination) {
		//if there is no connection
		if (getOutgoingConnections().size() == 0) {
			GeneriqueCimeroModel tmp = getDiagram().getGeneriqueModel(
					nameDestination);
			//if the generiquemodel exist
			if (tmp != null) {
				ConnectionCreateCommand newConnect =
					new ConnectionCreateCommand();
				newConnect.setSource(this);
				newConnect.setTarget(tmp);
				newConnect.execute();
				return true;
			} else {
				return false;
			}
		} else {
			Connection modelTmp = (Connection) getOutgoingConnections().get(0);
			if (nameDestination == modelTmp.getTarget().getName()) {
				//this is the same object
				return false;
			} else	{
				//this is a new object
				//first, we delete the current connection beetween this
				//object and the other
				modelTmp.getTarget().removeInput(modelTmp);
				removeOutput(modelTmp);
				//we retreive the generiqueModel destination
				GeneriqueCimeroModel tmp = getDiagram().getGeneriqueModel(
						nameDestination);
				//if the generiquemodel exist
				if (tmp != null) {
					ConnectionCreateCommand newConnect =
						new ConnectionCreateCommand();
					newConnect.setSource(this);
					newConnect.setTarget(tmp);
					newConnect.execute();
					return true;
				} else {
					if (nameDestination.compareTo("") == 0) {
						//the seleted destination is null, there is no pb
						return true;
					}
					//the destination doesn't exist so there is a problem
					return false;
				}
			}
		}
	}

	/**
	 * This function is used to retreive the Name object destination
	 * if there is no destination object, this function return "".
	 * @return the destination name
	 */
	protected final String getNameFirstDestination() {
		if (getOutgoingConnections().size() != 0) {
			Connection modelTmp = (Connection) getOutgoingConnections().get(0);
			if (modelTmp != null) {
				return modelTmp.getTarget().getName();
			}
		}
		return "";
	}

	//---------------------------------------------------------------------
	// HASHMAP function

	/**
	 * set the property value of the hashmap.
	 * @param id : the key of the hashmap
	 * @param value : the value of the property.
	 */
	public final void setProperty(final String id, final Object value) {
		if (propertiesMap.containsKey(id)) {
			propertiesMap.remove(id);
		}
		propertiesMap.put(id, value);
	}

	/**
	 * get the property identify by the parameter.
	 * @param id : the key of this object
	 * @return the value of this key or null if the key doesn't exist
	 */
	public final Object getProperty(final String id) {
		if (propertiesMap.containsKey(id)) {
			return propertiesMap.get(id);
		}
		return null;
	}
	
	public final Set<String> getPropertyKeys() {
		return propertiesMap.keySet();
	}

	//---------------------------------------------------------------------

	//----------------------------------------------------------------------
	// Abstract methods
	/**
	 * This function is used to validate the object.
	 * @param showLog : true if the log must be show in log view.
	 *@return boolean
	 */
	public abstract boolean isValide(boolean showLog);

	/**.
	 * This function is used to initalize the descriptor list
	 *
	 */
	protected abstract void createDescriptor();

	/**
	 * Return the default name of the Object.
	 * @return String
	 */
	private String getDefaultName() {
		return defaultName;
	}

	/**
	 * This function is used to know if the object has
	 * input connection.
	 * @return boolean
	 */
	public abstract boolean isInput();

	/**
	 * This function is used to know if the object has
	 * output connection.
	 * @return boolean
	 */
	public abstract boolean isOutput();

	/**
	 * @return the id of this element.
	 *
	 */
	public abstract String getComponentId();

	//----------------------------------------------------------------------
	// IPropertyDescriptor methods
	/**
	 * return the descriptor.
	 * @return IPropertyDescriptor[]
	 */
	public final IPropertyDescriptor[] getPropertyDescriptors() {
		//init the descriptor list
		descriptorList = new ArrayList<IPropertyDescriptor>();
		createDescriptor();
		//transform descriptor list in a IpropertyDescriptor[]
		IPropertyDescriptor [] myDescTable =
			new IPropertyDescriptor[descriptorList.size()];
		int indice = 0;
		for (IPropertyDescriptor it : descriptorList) {
			myDescTable[indice++] = it;
		}
		return myDescTable;
	}

	/**
	 * add a descriptor in the descriptor model list.
	 * @param myDescr : the descriptor to add in the descriptor list.
	 */
	public final void addDescriptor(final IPropertyDescriptor myDescr) {
		descriptorList.add(myDescr);
	}

	/**
	 * add a text property descriptor in a prpoerti descriptor list.
	 * @param id : the id of this descriptor.
	 * @param displayName : the name of this descriptor.
	 */
	protected final void addTextDescriptor(final String id,
			final String displayName) {
		descriptorList.add(new TextPropertyDescriptor(id, displayName));
	}

	/**
	 * add a text property descriptor in a prpoerti descriptor list.
	 * @param id : the id of this descriptor.
	 * @param displayName : the name of this descriptor.
	 */
	protected final void addTextDescriptor(final String id,
			final String displayName, final String category) {
		TextPropertyDescriptor pDescr = new TextPropertyDescriptor(id, displayName);
		((TextPropertyDescriptor)pDescr).setCategory(category);
		descriptorList.add(pDescr);
	}
	
	/**
	 * add a text property descriptor in a prpoerti descriptor list.
	 * @param id : the id of this descriptor.
	 * @param displayName : the name of this descriptor.
	 */
	protected final void addPasswordTextDescriptor(final String id,
			final String displayName) {
		descriptorList.add(new PasswordTextPropertyDescriptor(id, displayName));
	}
	/**
	 * add a combobox  property descriptor in the descriptor component list.
	 * @param id : the id of this descriptor.
	 * @param displayName : the display name
	 * @param value : the table value of this combo.
	 */
	protected final void addComboBoxDescriptor(final String id,
			final String displayName, final String [] value) {
		descriptorList.add(new ComboBoxPropertyDescriptor(
				id, displayName, value));
	}


	/**
	 * method called when a property is changed.
	 * @param id : object id
	 * @param value : object value
	 * @see org.eclipse.ui.views.properties.
	 * IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		if (id == NAME) {
			if (!shellOpen) {
				shellOpen = true;
				//we verify that the specify name doesn't already exist.
				if (nameExist((String) value)) {
					//the name is already use by another component
					Shell monShell = new Shell();
					MessageBox msg = new MessageBox(monShell , SWT.OK);
					msg.setMessage(
							"The name that you specify is already used !\n"
							+ "Please select another name.");
					msg.setText("Name existing.");
					//the messagebox is open
					msg.open();
				} else {
					//we verify that the selected name is not null.
					if (((String) value).compareTo("") == 0) {
						//the component name cannot be null.
						Shell monShell = new Shell();
						MessageBox msg = new MessageBox(monShell , SWT.OK);
						msg.setMessage(
								"The name of the component cannot be null.");
						msg.setText("Name problem.");
						//the messagebox is open
						msg.open();
					} else {
						setProperty(NAME, (String) value);
					}
				}
				shellOpen = false;
			}
		} else if (getProperty((String) id) == null) {
			//sent a log message if the id is not known
			SpagicLogger.warning("the id (" + id + ") is not known !"
					, new Exception());
		} else {
			//
			//set the property in the map.
			//
			setProperty((String) id, value);
		}
		//tell the parts, property change --> refresh
		fireStructureChange(NAME, this);
	}

	/**
	 * Return the value of a property.
	 * @param id : the id of the property
	 * @return Object
	 */
	public Object getPropertyValue(final Object id) {
		return getProperty((String) id);
	}
	/**
	 * Abstract IPropertySource method.
	 * It is used to determine the editable object
	 * @return Object
	 */
	public final Object getEditableValue() {
		return this;
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource
	 * #isPropertySet(java.lang.Object)
	 * @param id : the id of the property.
	 * @return true.
	 */
	public final boolean isPropertySet(final Object id) {
		//return true to enable the method resetPropertyValue()
		return true;
	}

	/**
	 * This function is used to research if the element in param
	 * has a BC in input.
	 * This function return true if the element has an input connection.
	 * @param elt : the element of reference
	 * @return boolean
	 */
	public final List < GeneriqueCimeroModel > researchInput(
			final GeneriqueCimeroModel elt) {
		//create the result list
		List < GeneriqueCimeroModel > resultLst =
			new ArrayList < GeneriqueCimeroModel >();
		//add to the result list all the predecesseur element
		for (int i = 0; i < elt.getIncomingConnections().size(); i++) {
			GeneriqueCimeroModel eltLst =
				((Connection) elt.getIncomingConnections().get(i)).getSource();
			//add the result of these childs
			List < GeneriqueCimeroModel > res = researchInput(eltLst);
			for (int j = 0; j < res.size(); j++) {
				if (!resultLst.contains(res.get(j))) {
					resultLst.add(res.get(j));
				}
			}
			//add this elt if it doesn't already exist
			if (!resultLst.contains(eltLst)) {
				resultLst.add(eltLst);
			}
		}
		return resultLst;
	}

	public boolean isCustomComponent() {
		return customComponent;
	}

	public void setCustomComponent(boolean customComponent) {
		this.customComponent = customComponent;
	}

	public String getCustomComponentId() {
		return customComponentId;
	}

	public void setCustomComponentId(String customComponentId) {
		this.customComponentId = customComponentId;
	}
	
	public void configureCustomComponent(){
		// DO NOTHING
	}
}
