package com.bull.cimero.pluginEditor.editors.model.SEModel;

import org.spagic.utils.SpagicLogger;


/**
 * The aggregator model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class RecListAggregator extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The xsl transformeur path.*/
	private static final String NAMEDEST = "destination service";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myRecListAggregator";
	/**the id of this element.*/
	private static final String ID = "RECLIST_AGGREGATOR";
	/**the aggregate envelope*/
	private static final String AGGREGATE_ENVOLPE="AGGREGATE_ENVOLPE".intern();
	/**the message envelope*/
	private static final String MESSAGE_ENVOLPE="MESSAGE_ENVOLPE".intern();
	
	private static final String DEFAULT_AGGREGATE_ENVOLPE="aggregate".intern();
	private static final String DEFAULT_MESSAGE_ENVOLPE="message".intern();

	/**
	 * The default constructor.
	 */
	public RecListAggregator() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(AGGREGATE_ENVOLPE, DEFAULT_AGGREGATE_ENVOLPE);
		setProperty(MESSAGE_ENVOLPE, DEFAULT_MESSAGE_ENVOLPE);
		setStandard(true);
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the transformer name cannot be null";
			returnResult = false;
		}
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n2 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		String aggregateEnvelopeName = (String)getProperty(AGGREGATE_ENVOLPE);
		if (aggregateEnvelopeName == null ||
				aggregateEnvelopeName.trim().equals("")) {
			
			msg += "\n3 - Aggregate envelope name has to be valorized!";
			return false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected final void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
		
		addTextDescriptor(AGGREGATE_ENVOLPE, indice++ + " - Aggregate envelope name");
		addTextDescriptor(MESSAGE_ENVOLPE, indice++ + " - Message envelope name");
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	public String getAggregateEnvelopeName() {
		return (String)getProperty(AGGREGATE_ENVOLPE);
	}
	
	public String getMessageEnvelopeName() {
		return (String)getProperty(MESSAGE_ENVOLPE);
	}
}
