package com.bull.cimero.pluginEditor.editors.model.SEModel;

import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.model.BCModel.TCPModelHelper;
/**
 * @author Nicola Buso (nicola.buso@eng.it)
 */

public class TCPPipeline extends ServiceEngineModel {
	private static final String ID = "TCPPIPELINE".intern();
	
	private static final String DEFAULTNAME = "myTCPEnricher";
	
	private static final String NAMEDEST = "destination service";
	
	private static final long serialVersionUID = 1L;
	
	private TCPModelHelper helper;


	public TCPPipeline() {
		super(DEFAULTNAME, 1, DEFAULTOUTIN);
		getHelper().setDefaultProperties(this);

		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setStandard(true);
	}
	
	private TCPModelHelper getHelper() {
		if (helper == null) {
			helper = new TCPModelHelper();
			helper.setInput(true);
			helper.setOutput(false);
			helper.setBindingComponent(false);
		}
		return helper;
	}

	@Override
	protected void createDescriptor() {
		getHelper().setDescriptors(this);
		int indice = 1;
		addTextDescriptor(NAME, getHelper().getString(indice++) + " - Name");
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST, getHelper().getString(indice++) + " - Select the destination service", lstStr);
		
	} 

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	@Override
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the enricher name cannot be null";
			returnResult = false;
		}

		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n4 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}

		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		boolean superResult = getHelper().isValide(this, showLog);
		return returnResult && superResult;
	}
	
	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (TCPModelHelper.BASE64_ENCODE.equals(id)) {
			Object value = getProperty(TCPModelHelper.BASE64_ENCODE);
			if (value == null || !(value instanceof Integer)) {
				setProperty(TCPModelHelper.BASE64_ENCODE, 0);
				return 0;
			}
		} else if (TCPModelHelper.BASE64_DECODE.equals(id)) {
			Object value = getProperty(TCPModelHelper.BASE64_DECODE);
			if (value == null || !(value instanceof Integer)) {
				setProperty(TCPModelHelper.BASE64_DECODE, 0);
				return 0;
			}
		}
		return super.getPropertyValue(id);
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	@Override
	public final String getComponentId() {
		return ID;
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public final void resetPropertyValue(final Object id) {
	}

}
