package com.bull.cimero.pluginEditor.editors.model.SEModel;

import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.commands.ConnectionCreateCommand;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class WireTap extends ServiceEngineModel {

	//
	// CONSTANTE
	//
	/**The default name.*/
	private static final String DEFAULTNAME = "myTracer";
	/**The id of this component. The id is used to get information in
	 * ressource wml file.*/
	private static final String ID = "Tracer";
	//
	// WireTap Properties
	//
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the destination service property.*/
	private static final String NAMEDESTCOPY = "destination copy service";

	//
	// variable
	//
	/** The connection between this rule and the destination element. */
	private Connection connDest = null;
	/** The connection between this rule and the destination element. */
	private Connection connCopy = null;

	/**
	 *
	 */
	private static final long serialVersionUID = 3476691753469052942L;

	/**
	 * the default constructor.
	 */
	public WireTap() {
		super(DEFAULTNAME, 2, DEFAULTOUTIN);
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDESTCOPY, "");
		setProperty(NAMEDEST, "");
		setStandard(true);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.editors.model.SEModel
	 * .ServiceEngineModel#createDescriptor()
	 */
	protected final void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name : ");
		addComboBoxDescriptor(NAMEDEST, indice++ + " - destination : ", lstStr);
		addComboBoxDescriptor(NAMEDESTCOPY, indice++
				+ " - destination copy : ", lstStr);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .SEModel.ServiceEngineModel#isValide(boolean)
	 * @param showLog : true if the log must to be show.
	 * @return true if this component is valide.
	 */
	public final boolean isValide(final boolean showLog) {
		boolean returnResult = true;
		String msg = "";
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the tracer name cannot be null";
			returnResult = false;
		}
		if ((getConn(false) == null)
				 || (!isConnectionExistInOut(getConn(false)))) {
			msg += "\n2 - destination => "
				+ "the destination service cannot be null";
			returnResult = false;
		}
		if ((getConn(true) == null)
				 || (!isConnectionExistInOut(getConn(true)))) {
			//no destination service
			msg += "\n3 - destination copy => "
				+ "the destination copy service cannot be null";
			returnResult = false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueCimeroModel#getComponentId()
	 * @return the id of this component.
	 */
	public final String getComponentId() {
		return ID;
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource
	 * #getPropertyValue(java.lang.Object)
	 * @param id : the id of the property.
	 * @return the value of the id.
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			String currentDestinationName = "";
			//we get the name of the connection
			if (connDest != null) {
				//we test if the connection is always reference
				//in the OutgoindConnection
				if (isConnectionExistInOut(connDest)) {
					//we get the name
					currentDestinationName =
						connDest.getTarget().getName();
				}
			}
			//return the num of the name  of the object
			String [] lstStr  = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(currentDestinationName) == 0) {
					return i;
				}
			}
		} else if (NAMEDESTCOPY.equals(id)) {
			String currentDestinationName = "";
			//we get the name of the connection
			if (connCopy != null) {
				//we test if the connection is always reference
				//in the OutgoindConnection
				if (isConnectionExistInOut(connCopy)) {
					//we get the name
					currentDestinationName =
						connCopy.getTarget().getName();
				}
			}
			//return the num of the name  of the object
			String [] lstStr  = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(currentDestinationName) == 0) {
					return i;
				}
			}
		} else {
			return super.getPropertyValue(id);
		}
		return null;
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource
	 * #resetPropertyValue(java.lang.Object)
	 * @param id : the id of the property.
	 */
	public void resetPropertyValue(final Object id) {

	}

	/**
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueCimeroModel#setPropertyValue(java.lang.Object,
	 * java.lang.Object)
	 * @param id : the id of the property.
	 * @param value : the value of this property.
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum] , false)) {
				System.out.println("Error connection in routeur model");
			}
		} else if (id == NAMEDESTCOPY) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum], true)) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}

		}
		super.setPropertyValue(id, value);
	}

	/**
	 * we surcharge this function because the routeur model hasn't the same
	 * functionnement of the others SE model.
	 * @param nameDestination : the name of the destination
	 * @param copy : true if this is the destination or false to the copy
	 * @return boolean
	 */
	protected final boolean refreshDestinationConnection(
			final String nameDestination, final boolean copy) {
		try {
			if (getConn(copy) != null) {
				// we remove this connection
				getConn(copy).getTarget().removeInput(getConn(copy));
				removeOutput(getConn(copy));
				setConn(null, copy);
			}
			//we create a new connection
			GeneriqueCimeroModel tmp = getDiagram()
					.getGeneriqueModel(nameDestination);
			//if the generiquemodel exist
			if (tmp != null) {
				setConn(createNewConnection(tmp, copy) , copy);
				return true;
			} else {
				//the user has selected ""
				return true;
			}
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * This function create a new connection between this object and
	 * the destination object. It return true if there is no problem
	 * during the connection creation.
	 * @param destination : the destination object
	 * @param typeConnection : the type of this connection.
	 * @return connection
	 */
	public final Connection createNewConnection(
			final GeneriqueCimeroModel destination,
			final boolean typeConnection) {
		try {
			ConnectionCreateCommand newConnect = new ConnectionCreateCommand();
			newConnect.setSource(this);
			newConnect.setTarget(destination);
			newConnect.setConnectionBottom(typeConnection);
			newConnect.execute();
			return newConnect.getConnection();
		} catch (Exception e) {
			SpagicLogger.error("exception during connection creation", e);
			return null;
		}
	}

	/**
	 *
	 * @param copy : true if the connection to return is the copy
	 * connection.
	 * @return one of the two connection of this component.
	 */
	public final Connection getConn(final boolean copy) {
		if (copy) {
			return this.connCopy;
		} else {
			return this.connDest;
		}
	}

	/**
	 * set the connection.
	 * @param conn : the new connection.
	 * @param copy : true if the connection to set is the copy destination.
	 */
	public final void setConn(final Connection conn, final boolean copy) {
		if (copy) {
			this.connCopy = conn;
		} else {
			this.connDest = conn;
		}
	}

}
