package it.eng.components.cimero;

import it.eng.utils.DataSourceGuiHelper;

import java.io.IOException;
import java.io.ObjectInputStream;

import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;


 /**
 * The Generic JDBC model class.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class GenericJdbcComponent extends ServiceEngineModel implements IRequireDataSourceComponent {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The xsl transformeur path.*/
	private static final String DATASOURCE = "Datasource";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	private static final String INOUTMEP = "in/out MEP";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myJDBC";
	
	/**the id of this element.*/
	private static final String ID = "JDBCCOMPONENT";
	
	private DataSourceGuiHelper dsHelper;
	
	/**
	 * The default constructor.
	 */
	public GenericJdbcComponent() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		dsHelper = new DataSourceGuiHelper();
		
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
	}
	
	public void configureCustomComponent() {
		setProperty(DATASOURCE, dsHelper.getDefaultDatasourceName());
	}

	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the transformer name cannot be null";
			returnResult = false;
		}
		

		
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		
		if (id == DATASOURCE){
			int intValue = ((Integer)value).intValue();
			setProperty(DATASOURCE, dsHelper.getIdByPosition(intValue));
			return;
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (DATASOURCE.equals(id)) {
			String propertyDs = (String)getProperty(DATASOURCE);
			if (propertyDs == null || propertyDs.trim().length() == 0){
				return 0;
			}else{
				return dsHelper.getPositionById(propertyDs);
			}
		}else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected  void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		
		String[] comboDs = dsHelper.getIdNames();
		
		addComboBoxDescriptor(DATASOURCE, indice++ +" - dataSource", comboDs);

		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	public final String getDataSource() {
		return (String) getProperty(DATASOURCE);
	}
	
	private void readObject(ObjectInputStream in)
		throws IOException, ClassNotFoundException {
		
		in.defaultReadObject();
		if (dsHelper != null) {
			dsHelper.init();
		} else {
			dsHelper = new DataSourceGuiHelper();
		}
		// check if the preconfigured datasource is available,
		// else set the default datasource
		String propertyDs = (String)getProperty(DATASOURCE);
		if (propertyDs == null || propertyDs.trim().length() == 0) {
			SpagicLogger.warning("Unable to find saved DATASOURCE configuration" +
				" setting the default one");
			setProperty(DATASOURCE, dsHelper.getDefaultDatasourceName());
		}
	}
}
