/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.metadb.processor.impl;

import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.StpIntermediateModel;
import org.spagic.utils.ImPreprocessor;
import org.spagic.utils.SpagicJBIImPreprocessor;


/**
 * JBI ServiceMix preprocessor
 *
 */
public class JBIProcessor extends BaseProcessor {
	
	private ImPreprocessor imPreprocessor = null;
	protected ImFactory factory = null;
	private static String MARK_AS_PREPROCESSED = "MARK_AS_PREPROCESSED";
	
	public JBIProcessor(String name, String binding, String technology) {
		super(name, binding, technology);
		this.imPreprocessor = new SpagicJBIImPreprocessor(binding);
		this.factory = ImFactory.eINSTANCE;
	}
	

	/**
	 * @see org.spagic.im.out.metadb.processor.IProcessor#preProcess(org.eclipse.stp.im.Process, StpIntermediateModel)
	 */
	public org.eclipse.stp.im.Process preProcess(org.eclipse.stp.im.Process process, StpIntermediateModel stpIntermediateModel) {
		String processToEnhnaceName = process.getName();
		
		if (!isAlreadyPreprocessed(stpIntermediateModel)){
			imPreprocessor.process(stpIntermediateModel);
			markAsPreprocessed(stpIntermediateModel);
		}

		for (Process aProcess : stpIntermediateModel.getProcessCollection().getProcesses()) {
			if (aProcess.getName().equalsIgnoreCase(processToEnhnaceName))
				return aProcess;
		}
		
		return null;

		
	}
	private boolean isAlreadyPreprocessed(StpIntermediateModel stpIntermediateModel){
		Property p = stpIntermediateModel.getProperties().get(MARK_AS_PREPROCESSED);
		return p != null;
	}
	
	private void markAsPreprocessed(StpIntermediateModel stpIntermediateModel){
		Property p = factory.createBasicProperty();
		p.setKey("MARK_AS_PREPROCESSED");
		((BasicProperty)p).setValue("TRUE");
		stpIntermediateModel.getProperties().put(MARK_AS_PREPROCESSED, p);
	}
	/*
	private static final String HTTP_BC_DESCRIPTION = "HTTP BC component added by Spagic Studio";
	private static final String TCP_BC_DESCRIPTION = "TCP BC component added by Spagic Studio";

	private static final String WEB_SERVICE_PIPELINE = "JBI-WebServicePipeline";
	private static final String TCP_IP_PIPELINE = "JBI-TCPIP-Pipeline";
	
	private static final String HTTP_IO_BC = "JBI-HTTPInputOutputBindingComponent";
	private static final String TCP_IO_BC = "JBI-TcpIp-InputOutputBindingComponent";
	
	private static final String REPLY = "JBI-Reply";
	
	*/
		/*
		ImFactory stpModelfactory = ImFactory.eINSTANCE;
		
		// List for storing new steps and transition to the end of
		// preprocessing to avoid ConcurrentModificationException
		List<Step> newStepsList = new ArrayList<Step>();
		List<Transition> newTransitionsList = new ArrayList<Transition>();
		
		List<Step> stepList = process.getSteps();
		
		// AZ
		List<Step> replyStepsToRemove = new ArrayList<Step>();
		ServiceBinding sb = null;
		Step additionalStep = null;
		Step destinationStep = null;
		Transition sourceTransition = null;
		Transition newTransition = null;
		ServiceBinding syncSB = stpModelfactory.createServiceBinding();
		syncSB.setName("JBI-Sync");
		BasicProperty runtimeIdProperty = stpModelfactory.createBasicProperty();
		runtimeIdProperty.setKey(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
		runtimeIdProperty.setValue(getRuntime());
		syncSB.getProperties().put(runtimeIdProperty.getKey(),
				runtimeIdProperty);
		

		stpIntermediateModel.getServicebindings().add(syncSB);
		
		
		
		// -- AZ
		boolean syncAdded = false;
		for (Step aStep : stepList) {
			sb = SpagicUtils.getRightStepBinding(aStep, getRuntime());
			if ((sb != null)
					&& (sb.getName().equalsIgnoreCase(
							HTTP_IO_BC) || (sb
							.getName()
							.equalsIgnoreCase(TCP_IO_BC)))) {
				
				// We must insert a sync component
			
				if (process.getSteps().size() > 3){
					
					// Add the sync and relocate the transitions
					sourceTransition = aStep.getSourceTransitions().get(0);
					
					
					destinationStep = sourceTransition.getTarget();
					process.getTransitions().remove(sourceTransition);
					aStep.getSourceTransitions().remove(sourceTransition);
					
					additionalStep = stpModelfactory.createStep();
					additionalStep.setName(aStep.getName()
						+ SpagicUtilsConstants.JBI_SYNC_SUFFIX);
					additionalStep.getStepbindings().add(syncSB);
					newStepsList.add(additionalStep);

					newTransition = stpModelfactory.createTransition();
					newTransition.setSource(aStep);
					newTransition.setTarget(additionalStep);
					process.getTransitions().add(newTransition);

					newTransition = stpModelfactory.createTransition();
					newTransition.setSource(additionalStep);
					newTransition.setTarget(destinationStep);
					process.getTransitions().add(newTransition);
					
				
					newTransition = stpModelfactory.createTransition();
					newTransition.setSource(additionalStep);
					newTransition.setTarget(aStep);
					process.getTransitions().add(newTransition);
					
					
					syncAdded = true;
					
				}else{
					// Add the sync and relocate the transitions
					sourceTransition = aStep.getSourceTransitions().get(0);
					destinationStep = sourceTransition.getTarget();
					
					
					newTransition = stpModelfactory.createTransition();
					newTransition.setSource(destinationStep);
					newTransition.setTarget(aStep);
					process.getTransitions().add(newTransition);
					
					syncAdded = false;
					
				}
			}
		}
		
		// Change the transitions of the steps that has target reply to sync
		for (Step aStep : process.getSteps()) {
			List<Transition> trToRemoveFromStepSourceTransition = new ArrayList<Transition>();
			for (Transition tr : aStep.getSourceTransitions()) {
				sb = SpagicUtils.getRightStepBinding(tr.getTarget(), getRuntime());
				if (sb.getName().equalsIgnoreCase(REPLY)){
					replyStepsToRemove.add(tr.getTarget());
					if (syncAdded)
						tr.setTarget(additionalStep);
					else{
						process.getTransitions().remove(tr);
						trToRemoveFromStepSourceTransition.add(tr);
					}
				}
			}//for (Transition tr :
			aStep.getSourceTransitions().removeAll(trToRemoveFromStepSourceTransition);
		}//for (Step aStep :
		
		for (Step step : stepList) {
			// Retrieve the suitable binding for JBI
			ServiceBinding serviceBinding = SpagicUtils.getRightStepBinding(step, getRuntime());
			
			if (serviceBinding == null) {
				SpagicLogger.error(IM2MetaDBActivator.PLUGIN_ID,"Unable to retrieve service binding for runtime " + getRuntime(), null);
				return process;
			}
			
			// Check if the binding is related to a special case
			if (serviceBinding.getName().equals(WEB_SERVICE_PIPELINE) ||
				serviceBinding.getName().equals(TCP_IP_PIPELINE)) {
				boolean isHttp = serviceBinding.getName().equals(WEB_SERVICE_PIPELINE);
				
				// Create a new component for the HTTP/TCP BC
				Step bcStep = stpModelfactory.createStep();
				bcStep.setName(step.getName() + SpagicUtilsConstants.JBI_TRANSFORMER_SUFFIX);
				bcStep.setDescription(isHttp ? HTTP_BC_DESCRIPTION : TCP_BC_DESCRIPTION);
				
				// Create two transition (in and out) from the Enricher step to the new component
				Transition inTransition = stpModelfactory.createTransition();
				Transition outTransition = stpModelfactory.createTransition();
				
				inTransition.setSource(step);
				inTransition.setTarget(bcStep);
				
				outTransition.setSource(bcStep);
				outTransition.setTarget(step);
				
				// Add the new step and transitions to the process
				newStepsList.add(bcStep);
				newTransitionsList.add(inTransition);
				newTransitionsList.add(outTransition);				
			} 
		}
		
		
		process.getSteps().removeAll(replyStepsToRemove);
		process.getSteps().addAll(newStepsList);
		process.getTransitions().addAll(newTransitionsList);
	
		/*
		for (Step step : newStepsList) {
			process.getSteps().add(step);
		}
		for (Transition transition : newTransitionsList) {
			process.getTransitions().add(transition);
		}
		
		
		return process;
		
	}
	*/

}
